import os.path
import re
import sys
from urllib.parse import unquote
import numpy
import win32gui
from PIL import ImageGrab, Image, ImageQt
from airclick.utils import screen_utils
from airclick.daos.screen_daos import Point, OcrRes
import numpy as np

from airclick.utils.screen_utils import compare_colors
from airclick.windows.system import R
from airclick.utils import airclickcv as aircv


def capture(hwnd: int = 0, rect=None) -> Image:
    temp_img = None
    if hwnd != 0:
        temp_rect = win32gui.GetWindowRect(hwnd)
        temp_img = ImageGrab.grab(temp_rect)
    else:
        temp_img = ImageGrab.grab()

    if rect:
        temp_img = temp_img.crop(rect)
    return temp_img


def get_color(x, y):
    img = capture()
    return img.getpixel((x, y))


class FindColors:
    def __init__(self, colors: str = None, rect=[], sim: float = 0.9, ore: int = 2, space: int = 5, file: str = None,
                 dict_p: dict = None):
        self.rect = rect
        self.colors = colors
        self.sim = sim
        self.ore = ore
        self.space = space
        self.res_num = 1
        self.file = file

        if dict_p:
            if 'colors' in dict_p:
                self.colors = dict_p['colors']

            if 'rect' in dict_p:
                self.rect = dict_p['rect']

            if 'sim' in dict_p:
                self.sim = float(dict_p['sim'])

            if 'ore' in dict_p:
                self.ore = int(dict_p['ore'])

            if 'space' in dict_p:
                self.space = int(dict_p['space'])

            if 'file' in dict_p:
                self.file = unquote(dict_p['file'])

            if 'res_num' in dict_p:
                self.res_num = int(dict_p['res_num'])

    def find(self):
        self.res_num = 1
        res = self.exec_find()
        if res and len(res) > 0:
            return res[0]
        return None

    def find_all(self, res_num=sys.maxsize):
        self.res_num = res_num
        res = self.exec_find()
        if res or len(res) > 0:
            return res
        return None

    def exec_find(self):
        # 获取截图
        img = None
        if self.file:
            img = Image.open(self.file)
        else:
            img = ImageGrab.grab()

        if self.rect and len(self.rect) == 4:
            img = img.crop(self.rect)

        # 回填 坐标
        ps = screen_utils.find_colors(self.colors, self.sim, self.res_num, self.ore, img, self.space)
        if ps and self.rect and len(self.rect) == 4:
            for i in range(len(ps)):
                ps[i] = Point(ps[i].x + self.rect[0], ps[i].y + self.rect[1])

        return ps


class CompareColors:
    def __init__(self, colors: str = None, sim: float = 0.9, img_file: str = None, dict_p: dict = None):
        self.colors = colors
        self.sim = sim
        self.img_file = img_file
        if dict_p:
            if 'colors' in dict_p:
                self.colors = dict_p['colors']

            if 'sim' in dict_p:
                self.sim = float(dict_p['sim'])

            if 'path' in dict_p:
                self.img_file = unquote(dict_p['path'])

    def compare(self):
        if self.img_file:
            img = Image.open(self.img_file)
        else:
            img = capture()

        return compare_colors(self.colors, self.sim, img)


class Ocr:
    def __init__(self, rect=[], pattern: str = None, confidence: float = 0.1, img_path: str = None,
                 dict_p: dict = None):

        self.rect = rect
        self.pattern = pattern
        self.confidence = confidence
        self.img_path = img_path
        # self.ocr = PaddleOCR(ocr_version='PP-OCRv3')

        if dict_p:
            if 'rect' in dict_p:
                self.rect = dict_p['rect']

            if 'pattern' in dict_p:
                self.pattern = dict_p['pattern']

            if 'img_path' in dict_p:
                self.img_path = unquote(dict_p['img_path'])

            if 'confidence' in dict_p:
                self.sim = float(dict_p['confidence'])

            if 'findtype' in dict_p:
                if dict_p['findtype'] == 'find':
                    self.res_num = 1
                else:
                    self.res_num = 0

        # from easyocr import Reader
        #
        # self.ocr = Reader(['en', 'ch_sim'])

        from paddleocr import PaddleOCR
        self.ocr = PaddleOCR()

    def find(self) -> OcrRes:
        self.res_num = 1
        res = self.exec_find_paddle()
        if res:
            return res[0]
        return None

    def find_all(self, res_num: int = 0) -> [OcrRes]:
        self.res_num = res_num
        res = self.exec_find_paddle()
        return res

    def exec_find(self):
        if self.img_path:
            img = Image.open(self.img_path)
        else:
            img = capture(rect=self.rect)

        if self.rect and len(self.rect) == 4:
            img = img.crop(self.rect)

        # res_source = self.ocr.readtext(np.array(img))

        res_source = self.ocr.ocr(np.array(img))

        print(res_source)

        res = []
        if res_source:
            for r in res_source:
                if self.pattern:
                    pattern = re.compile(self.pattern)
                if r[2] >= self.confidence and (self.pattern is None or pattern.match(r[1])):

                    offsetx = 0
                    offsety = 0
                    if self.rect:
                        offsetx = self.rect[0]
                        offsety = self.rect[1]

                    region_position = r[0]
                    region_position = [[int(region_position[0][0]) + offsetx, int(region_position[0][1]) + offsety],
                                       [int(region_position[1][0]) + offsetx, int(region_position[1][1]) + offsety],
                                       [int(region_position[2][0]) + offsetx, int(region_position[2][1]) + offsety],
                                       [int(region_position[3][0]) + offsetx, int(region_position[3][1]) + offsety]]

                    if self.res_num == 0:
                        res.append(OcrRes(text=r[1], confidence=r[2], region_position=region_position))
                    elif len(res) < self.res_num:
                        res.append(OcrRes(text=r[1], confidence=r[2], region_position=region_position))
                    else:
                        return res

        return res

    def exec_find_paddle(self):
        if self.img_path:
            img = Image.open(self.img_path)
        else:
            img = capture(rect=self.rect)

        if self.rect and len(self.rect) == 4:
            img = img.crop(self.rect)

        res_source = self.ocr.ocr(np.array(img))
        res = []
        if res_source and res_source[0]:
            for r in res_source[0]:

                if self.pattern:
                    pattern = re.compile(self.pattern)
                    # [ [[238.0, 12.0], [330.0, 12.0], [330.0, 40.0], [238.0, 40.0]], ('版本控制', 0.9998359680175781)]

                if r[1][1] >= self.confidence and (self.pattern is None or pattern.match(r[1][0])):

                    offsetx = 0
                    offsety = 0
                    if self.rect:
                        offsetx = int(self.rect[0])
                        offsety = int(self.rect[1])

                    region_position = r[0]
                    region_position = [[int(region_position[0][0]) + offsetx, int(region_position[0][1]) + offsety],
                                       [int(region_position[1][0]) + offsetx, int(region_position[1][1]) + offsety],
                                       [int(region_position[2][0]) + offsetx, int(region_position[2][1]) + offsety],
                                       [int(region_position[3][0]) + offsetx, int(region_position[3][1]) + offsety]]

                    if self.res_num == 0:
                        res.append(OcrRes(text=r[1][0], confidence=r[1][1], region_position=region_position))
                    elif len(res) < self.res_num:
                        res.append(OcrRes(text=r[1][0], confidence=r[1][1], region_position=region_position))
                    else:
                        return res

        return res

    def to_dict(self):
        pass


class FindImages:

    def __init__(self, img_part: str = None, source_rect=None, source_file: str = None, confidence: float = 0.1,
                 dict_p: dict = None):

        self.img_part = img_part
        self.source_rect = source_rect
        self.source_file = source_file
        self.confidence = confidence

        if dict_p:
            if 'image' in dict_p:
                self.img_part = dict_p['image']

            if 'screen' in dict_p:
                self.source_file = unquote(dict_p['screen'])

            if 'sim' in dict_p:
                self.confidence = float(dict_p['sim'])
        else:
            if not os.path.exists(self.img_part):
                self.img_part = R().file("res/img/" + img_part)
                if not os.path.exists(self.img_part):
                    raise ValueError('图片路径不存在:' + self.img_part)

        self.source_img = numpy.array(self.__get_source_img())
        self.search_img = numpy.array(Image.open(self.img_part))

        # self.source_img = cv2.imread("D:\\work\\test\\ac_client\\assets\\tools\\capture\\1703126140.png")
        # self.search_img = cv2.imread("D:\\work\\test\\res\\img\\c.png")

    def find(self, various_size=False):

        res = self.find_template()
        if res:
            return res

        res = self.find_sift()

        return res

    def find_all(self, maxcnt: int = 0, various_size=False):
        res = self.find_all_template()
        if res and len(res)>0:
            return res

        res = self.find_all_sift()
        return res

    def find_sift(self, min_match_count: int = 4):
        res = self.find_all_sift(maxcnt=1)
        if not res:
            return None
        return res[0]

    def find_all_sift(self, min_match_count: int = 4, maxcnt=0):
        res = aircv.find_all_sift(self.source_img, self.search_img, min_match_count, maxcnt)

        f_res = []
        for r in res:
            if r['confidence'] >= self.confidence:
                f_res.append(r)

        return f_res

    # def find_all_sift_gpt(self, min_match_count: int = 4, maxcnt=0):
    #     return aircv.find_all_sift_gpt(self.source_img, self.search_img, min_match_count, maxcnt)

    def find_template(self, threshold=0.5, rgb=False, bgremove=False):
        f_res = self.find_all_template(maxcnt=1, threshold=threshold, rgb=rgb, bgremove=bgremove)
        if f_res and len(f_res) > 0:
            return f_res[0]
        return None

    def find_all_template(self, maxcnt=0, threshold=0.5, rgb=False, bgremove=False):

        res = aircv.find_all_template(self.source_img, self.search_img, threshold, maxcnt, rgb, bgremove)

        f_res = []
        for r in res:
            if r['confidence'] >= self.confidence:
                f_res.append(r)

        return f_res

    def __get_source_img(self):
        temp_img = None
        if self.source_file:
            temp_img = Image.open(self.source_file)
        else:
            temp_img = capture()

        if self.source_rect and self.source_img.size == 4:
            temp_img = temp_img.resize(self.source_rect)

        return temp_img



import ctypes
import sys
import pyautogui
import win32con
import win32gui
from PIL import ImageGrab, Image, ImageQt
import re

from PyQt5.QtWidgets import QApplication

app = QApplication(sys.argv)


class Window:

    @staticmethod
    def find(active: bool = True, title: str = None, name: str = None, hwnd: int = None):

        if hwnd:
            return Window(hwnd)

        window_list = Window.find_all(active, title, name)
        if window_list and len(window_list) > 0:
            return window_list[0]

        return None

    @staticmethod
    def find_all(active: bool = None, title: str = None, name: str = None):
        window_list = []

        def tempcall(hwnd, list):
            window_list.append(Window(hwnd, False))

        win32gui.EnumWindows(tempcall, window_list)

        f_window_list = []
        for w in window_list:

            if active is not None:
                if active != w.is_active():
                    continue

            if title is not None:
                if not w.__base_re(title, w.title):
                    continue

            if name is not None:
                if not w.__base_re(name, w.title):
                    continue

            f_window_list.append(w)

        return f_window_list

    @staticmethod
    def active():
        return Window(win32gui.GetForegroundWindow())

    @staticmethod
    def screen_size():
        return pyautogui.size()

    def __init__(self, hwnd: int, fill_children: bool = True):
        self.qapp = None
        self.children = None
        self.hwnd = hwnd
        self.title = win32gui.GetWindowText(hwnd)
        self.rect = win32gui.GetWindowRect(hwnd)
        self.name = win32gui.GetClassName(hwnd)
        # 是否最小化
        if fill_children:
            self.__fill_childs(hwnd)

    def __fill_childs(self, hwnd: int):
        self.children = []

        def node_enum(hwnd, extra):
            self.children.append(Window(hwnd))

        win32gui.EnumChildWindows(hwnd, node_enum, None)

    def __str__(self):
        return {'hwnd':self.hwnd,'title':self.title,'name':self.name,'rect':self.rect,'active':self.is_active()}.__str__()

    def to_dict(self):
        tempchilds = []

        if self.children:
            for cnode in self.children:
                tempchilds.append(cnode.to_dict())

        return {
            'hwnd': self.hwnd,
            'name': self.name,
            'title': self.title,
            'rect': self.rect,
            'children': tempchilds,
            "placement": self.placement(),
            # "is_maximized": self.is_maximized(),
            "is_active": self.is_active(),
            "has_focus": self.has_focus(),
        }

    def capture(self) -> Image:
        return pyautogui.screenshot()

    def views(self):
        pass

    def is_active(self):
        return win32gui.GetForegroundWindow() == self.hwnd

    def has_focus(self):
        return win32gui.GetFocus() == self.hwnd

    def placement(self):
        # return ctypes.windll.user32.IsIconic(self.hwnd) != 0
        return win32gui.GetWindowPlacement(self.hwnd)[0]

    def frame(self, x=None, y=None, w=None, h=None):

        if x is None:
            x = self.rect[0]

        if y is None:
            y = self.rect[1]

        if w is None:
            w = self.rect[2] - self.rect[0]

        if h is None:
            h = self.rect[3] - self.rect[1]

        win32gui.MoveWindow(self.hwnd, int(x), int(y), int(w), int(h), True)

    def mize_mini(self):
        win32gui.ShowWindow(self.hwnd, win32con.SW_MINIMIZE)

    def mize_max(self):
        win32gui.ShowWindow(self.hwnd, win32con.SW_MAXIMIZE)

    def mize_normal(self):
        win32gui.ShowWindow(self.hwnd, win32con.SW_NORMAL)

    def close(self):
        win32gui.SendMessage(self.hwnd, win32con.WM_CLOSE, 0, 0)

    def __base_re(self, pattern, value):
        if not re.match(pattern, value):
            return False

        return True


class Selector:

    def __init__(self, model: int = 0, dict=None, ):
        self.find_max = -1
        self.model = model
        self.sels = []

        # 初始化json
        if dict:
            self.model = int(dict['model'])
            self.sels = dict['sels']
            self.find_max = dict['find']

    def find(self):
        self.find_max = 1

        find_res = self.exec_find()

        if find_res and len(find_res) > 0:
            return find_res[0]

        return None

    def find_all(self, num: int = sys.maxsize):
        self.find_max = -1
        return self.exec_find()

    def exec_find(self):
        window_list = []

        if self.model <= 0:
            def tempcall(hwnd, list):
                if self.model == 0:
                    window_list.append(Window(hwnd, False))
                elif self.model == -1:
                    window_list.append(Window(hwnd))

            win32gui.EnumWindows(tempcall, window_list)
        else:
            window_list.append(Window(self.model))

        return self.__check(window_list)

    def title(self, title: str):
        self.sels.append({"key": "title", "value": title})
        return self

    def name(self, name: str):
        self.sels.append({"key": "name", "value": name})
        return self

    def hwnd(self, hwnd: str):
        self.sels.append({"key": "hwnd", "value": hwnd})
        return self

    def __check(self, node_list):
        if len(self.sels) < 1:
            return node_list

        res_nodes = []
        for node in node_list:
            for sel in self.sels:
                if self.find_max > 0 and len(res_nodes) >= self.find_max:
                    return res_nodes

                key = sel['key']
                vales = sel['value']
                if key == "title":
                    if self.__base_re(vales, node.title):
                        res_nodes.append(node)

                if key == "name":
                    if self.__base_re(vales, node.name):
                        res_nodes.append(node)

                if key == "hwnd":
                    if self.__base_re(vales, node.hwnd):
                        res_nodes.append(node)

        return res_nodes


